@@ -1,5 +1,6 @@
 /*
  *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -21,7 +22,6 @@
 #define FBIO_H
 
 #include "io.h"
-#include "hash.h"
 
 class FbIoDispatcher : public IoDispatcher {
 public:
@@ -36,7 +36,6 @@ private:
 	virtual void removeIoSource(IoPipe *src, bool isread);
 
 	s32 mEpollFd;
-	HashTable<IoPipe*> *mpIoSources;
 };
 
 #endif
