@@ -1,5 +1,6 @@
 /*
  *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -32,6 +33,7 @@ public:
 	}
 	void processSysKey(u32 key);
 	void processSignal(u32 signo);
+	void initChildProcess();
 
 private:
 	void init();
