@@ -1,5 +1,6 @@
 /*
  *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -26,11 +27,16 @@
 class TtyInput : public IoPipe {
 	DECLARE_INSTANCE(TtyInput)
 public:
-	void enterLeaveVc(bool enter);
+	void switchVc(bool enter);
+	void switchIm(bool enter, bool raw);
 
 private:
 	virtual void readyRead(s8 *buf, u32 len);
 	void setupSysKey(bool restore);
+	void processRawKeys(s8* buf, u32 len);
+	
+	bool mRawMode;
+	bool mImEnable;
 };
 
 #endif
