@@ -1,5 +1,6 @@
 /*
  *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -23,9 +24,11 @@
 typedef char s8;
 typedef short s16;
 typedef int s32;
+typedef long long s64;
 
 typedef unsigned char u8;
 typedef unsigned short u16;
 typedef unsigned int u32;
+typedef unsigned long long u64;
 
 #endif
