@@ -1,5 +1,6 @@
 /*
  *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   This file is part of FbTerm.
  *   based on GTerm by Timothy Miller <tim@techsource.com>
  *
  *   This program is free software; you can redistribute it and/or
@@ -20,24 +21,24 @@
 
 #include <string.h>
 #include "vterm.h"
-#include "io.h"
 
 #define MIN(a, b) ((a) < (b) ? (a) : (b))
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 
-const VTerm::CharAttr VTerm::default_char_attr = { -1, -1, 1, 0, 0, 0, 0, VTerm::CharAttr::Single };
+VTerm::CharAttr VTerm::default_char_attr = { 0, 0, 1, 0, 0, 0, 0, VTerm::CharAttr::Single };
 
 VTerm::CharAttr VTerm::normal_char_attr()
 {
 	CharAttr a(char_attr);
-	if (a.underline && default_underline_color != -1) {
+
+	if (a.underline && cur_underline_color != -1) {
 		a.underline = false;
-		a.fcolor = default_underline_color;
+		a.fcolor = cur_underline_color;
 	}
 
-	if (a.intensity == 0 && default_halfbright_color != -1) {
+	if (a.intensity == 0 && cur_halfbright_color != -1) {
 		a.intensity = 1;
-		a.fcolor = default_halfbright_color;
+		a.fcolor = cur_halfbright_color;
 	}
 
 	return a;
@@ -46,6 +47,7 @@ VTerm::CharAttr VTerm::normal_char_attr()
 VTerm::CharAttr VTerm::erase_char_attr()
 {
 	CharAttr a(default_char_attr);
+
 	a.fcolor = char_attr.fcolor;
 	a.bcolor = char_attr.bcolor;
 	a.blink = char_attr.blink;
@@ -72,8 +74,8 @@ void VTerm::init_state()
 	do { \
 		bzero(hash_##a, sizeof(hash_##a)); \
 		for (u32 i = 0; a[i].key != (u16)-1; i++) { \
-			for (u32 j = 0; j <= (a[i].key >> 8); j++) { \
-				hash_##a[(a[i].key & 0xff) + j] = a + i; \
+			for (u32 j = 0; j <= (u32)(a[i].key >> 8); j++) { \
+				hash_##a[(u32)(a[i].key & 0xff) + j] = a + i; \
 			} \
 		} \
 	} while (0)
@@ -90,6 +92,8 @@ VTerm::VTerm(u16 w, u16 h)
 		inited = true;
 		init_state();
 		history_lines = init_history_lines();
+		default_char_attr.fcolor = init_default_color(true);
+		default_char_attr.bcolor = init_default_color(false);
 	}
 
 	text = 0;
@@ -124,7 +128,7 @@ VTerm::~VTerm()
 
 void VTerm::reset()
 {
-	utf8 = !strcasecmp(IoPipe::localCodec(), "UTF-8");
+	utf8 = true;
 	utf8_count = 0;
 //	g0_is_active = true;
 	normal_state = true;
@@ -137,13 +141,17 @@ void VTerm::reset()
 
 	mode_flags = ModeFlag();
 	char_attr = s_char_attr = default_char_attr;
-	default_fcolor = default_bcolor = -1;
-	default_underline_color = default_halfbright_color = -1;
+	cur_fcolor = default_char_attr.fcolor;
+	cur_bcolor = default_char_attr.bcolor;
+	cur_underline_color = -1;
+	cur_halfbright_color = -1;
 
 	if (text) {
 		bzero(tab_stops, max_width / 8 + 1);
 		clear_area(0, 0, width - 1, height - 1);
 	}
+
+	modeChanged(AllModes);
 }
 
 void VTerm::resize(u16 w, u16 h)
@@ -196,7 +204,7 @@ void VTerm::resize(u16 w, u16 h)
 
 		if (text) {
 			u32 start, new_start;
-			u16 history_copy_lines = history_full ? history_lines : history_save_line;
+			u16 history_copy_lines = (history_full ? history_lines : history_save_line);
 			for (u16 i = 0; i < history_copy_lines; i++) {
 				start = i * max_width;
 				new_start = i * new_max_width;
@@ -204,7 +212,7 @@ void VTerm::resize(u16 w, u16 h)
 				memcpy(&new_attrs[new_start], &attrs[start], sizeof(*attrs) * max_width);
 
 				for (u16 j = max_width; j < new_max_width; j++) {
-					new_text[new_start + j] = 0x20;
+					new_text[new_start + j] = ' ';
 					new_attrs[new_start + j] = default_char_attr;
 				}
 			}
@@ -227,6 +235,7 @@ void VTerm::resize(u16 w, u16 h)
 	}
 
 	bool h_changed = false;
+	
 	if (h != height) {
 		h_changed = true;
 
@@ -246,7 +255,7 @@ void VTerm::resize(u16 w, u16 h)
 	height = h;
 	
 	if (h_changed) {
-		historyChange(visual_start_line, total_history_lines());
+		historyChanged(visual_start_line, total_history_lines());
 	}
 	
 	scroll_bot = height - 1;
@@ -265,10 +274,14 @@ void VTerm::resize(u16 w, u16 h)
 void VTerm::input(const u8 *buf, u32 count)
 {
 	if (!width) return;
-
+	
 	if (visual_start_line != total_history_lines()) {
 		historyDisplay(true, total_history_lines());
-		historyChange(visual_start_line, total_history_lines());
+		historyChanged(visual_start_line, total_history_lines());
+	}
+
+	if (mode(CursorVisible)) {
+		changed_line(cursor_y, cursor_x, cursor_x);
 	}
 
 	u32 c, tc;
@@ -373,7 +386,7 @@ void VTerm::input(const u8 *buf, u32 count)
 		 * them; to display an arbitrary font position use the
 		 * direct-to-font zone in UTF-8 mode.
 		 */
-		bool ok = tc && (c >= 32 || !(mode_flags.display_ctrl ? (CTRL_ALWAYS >> c) & 1 : utf8 || ((CTRL_ACTION >> c) & 1)))
+		bool ok = tc && (c >= 32 || !(mode_flags.display_ctrl ? ((CTRL_ALWAYS >> c) & 1) : (utf8 || ((CTRL_ACTION >> c) & 1))))
 				  && (c != 127 || mode_flags.display_ctrl)
 				  && (c != 128+27);
 
@@ -396,6 +409,7 @@ void VTerm::input(const u8 *buf, u32 count)
 	}
 
 	update();
+	draw_cursor();
 }
 
 /* is_double_width() is based on the wcwidth() implementation by
@@ -446,7 +460,7 @@ void VTerm::do_normal_char()
 	if ((!dw && cursor_x >= width) || (dw && (cursor_x >= width - 1))) {
 		if (mode_flags.auto_wrap) {
 			if (dw && cursor_x == width - 1) {
-				text[yp + cursor_x] = 0x20;
+				text[yp + cursor_x] = ' ';
 				attrs[yp + cursor_x] = erase_char_attr();
 			}
 			next_line();
@@ -563,14 +577,12 @@ void VTerm::update()
 			dirty_endx[i] = 0;
 		}
 	}
-
-	draw_cursor();
 }
 
 void VTerm::draw_cursor()
 {
-	if (!mode_flags.cursor_visible) return;
-	
+	if (!mode(CursorVisible)) return;
+
 	u32 yp = linenumbers[cursor_y] * max_width + cursor_x;
 
 	CharAttr attr = attrs[yp];
@@ -610,7 +622,7 @@ void VTerm::expose(u16 x, u16 y, u16 w, u16 h)
 		
 			if (attrs[yp + cur] != attr) {
 				attr.reverse ^= mode_flags.inverse_screen;
-				drawChars(attr, start, y, num, codes, dws);
+				drawChars(attr, start, y, cur - start, num, codes, dws);
 
 				num = 0;
 				start = cur;
@@ -622,7 +634,25 @@ void VTerm::expose(u16 x, u16 y, u16 w, u16 h)
 		}
 
 		attr.reverse ^= mode_flags.inverse_screen;
-		drawChars(attr, start, y, num, codes, dws);
+		drawChars(attr, start, y, cur - start, num, codes, dws);
+	}
+}
+
+void VTerm::inverse(u16 sx, u16 sy, u16 ex, u16 ey)
+{
+	if (sy > ey) return;
+	if (charAttr(sx, sy).type == CharAttr::DoubleRight) sx--;
+	if (charAttr(ex, ey).type == CharAttr::DoubleLeft) ex++;
+	if (sy == ey && sx > ex) return;
+	
+	for (u16 y = sy; y <= ey; y++) {
+		u32 yp = get_line(y) * max_width;
+		u16 start = (y == sy) ? sx : 0;
+		u16 end = (y == ey) ? ex : (w() - 1);
+
+		for (u16 x = start; x <= end; x++) {
+			attrs[yp + x].reverse ^= 1;
+		}
 	}
 }
 
@@ -664,17 +694,18 @@ void VTerm::scroll_region(u16 start_y, u16 end_y, s16 num)
 
 			linenumbers[y] = temp[takey];
 
-			if (!fast_scroll || clr) {
-				dirty_startx[y] = 0;
-				dirty_endx[y] = width-1;
-			} else {
-				dirty_startx[y] = temp_sx[takey];
-				dirty_endx[y] = temp_ex[takey];
-			}
-			if (clr) {
-				clear_area(0, y, width - 1, y);
-			}
-		}
+            if (!fast_scroll || clr) {
+                dirty_startx[y] = 0;
+                dirty_endx[y] = width-1;
+            } else {
+                dirty_startx[y] = temp_sx[takey];
+                dirty_endx[y] = temp_ex[takey];
+            }
+ 
+            if (clr) {
+                clear_area(0, y, width - 1, y);
+            }
+ 		}
 	}
 }
 
@@ -701,7 +732,7 @@ void VTerm::shift_text(u16 y, u16 start_x, u16 end_x, s16 num)
 	u16 x = (num < 0) ? (num + end_x + 1) : start_x;
 	if (num < 0) num = -num;
 	for (; num--; x++) {
-		text[yp + x] = 0x20;
+		text[yp + x] = ' ';
 		attrs[yp + x] = erase_char_attr();
 	}
 
@@ -720,7 +751,7 @@ void VTerm::clear_area(u16 start_x, u16 start_y, u16 end_x, u16 end_y)
 	for (y=start_y; y<=end_y; y++) {
 		yp = linenumbers[y]*max_width;
 		for (x=start_x; x<=end_x; x++) {
-			text[yp+x]= 0x20;
+			text[yp+x]= ' ';
 			attrs[yp+x] = erase_char_attr();
 		}
 		changed_line(y, start_x, end_x);
@@ -740,32 +771,33 @@ void VTerm::changed_line(u16 y, u16 start_x, u16 end_x)
 
 void VTerm::move_cursor(u16 x, u16 y)
 {
-	changed_line(cursor_y, cursor_x, cursor_x);
-	
 	if (x>=width) x = width-1;
 	if (y>=height) y = height-1;
 	cursor_x = x;
 	cursor_y = y;
 }
 
-bool VTerm::mode(ModeType type)
+u16 VTerm::mode(ModeType type)
 {
-	bool ret = false;
+	u16 ret = false;
 	switch (type) {
 	case AutoRepeatKey:
 		ret = mode_flags.autorepeat_key;
 		break;
-	case NumericKeypad:
-		ret = mode_flags.numeric_keypad;
+	case ApplicKeypad:
+		ret = mode_flags.applic_keypad;
 		break;
-	case X10MouseReport:
-		ret = mode_flags.x10_mouse_report;
+	case MouseReport:
+		ret = mode_flags.mouse_report;
 		break;
-	case X11MouseReport:
-		ret = mode_flags.x11_mouse_report;
+	case CursorShape:
+		ret = mode_flags.cursor_shape;
 		break;
-	case CursorKeyEsc0:
-		ret = mode_flags.cursorkey_esc0;
+	case CursorKeyEscO:
+		ret = mode_flags.cursorkey_esco;
+		break;
+	case CRWithLF:
+		ret = mode_flags.crlf;
 		break;
 	case CursorVisible:
 		ret = mode_flags.cursor_visible && (visual_start_line == total_history_lines());
@@ -790,7 +822,7 @@ void VTerm::history_scroll(u16 num)
 
 		if (width < max_width) {
 			for (u16 i = width; i < max_width; i++) {
-				text[yp_history + i] = 0x20;
+				text[yp_history + i] = ' ';
 				attrs[yp_history + i] = default_char_attr;
 			}
 		}
@@ -803,7 +835,7 @@ void VTerm::history_scroll(u16 num)
 	}
 
 	visual_start_line = total_history_lines();
-	historyChange(visual_start_line, total_history_lines());
+	historyChanged(visual_start_line, total_history_lines());
 }
 
 void VTerm::historyDisplay(bool absolute, s32 num)
@@ -822,7 +854,7 @@ void VTerm::historyDisplay(bool absolute, s32 num)
 	if (visual_start_line > total_history_lines()) visual_start_line = total_history_lines();
 	if (visual_start_line == bak_line) return;
 	
-	modeChange(CursorVisible);
+	modeChanged(CursorVisible);
 
 	bool accel_scroll = false;
 
@@ -849,6 +881,8 @@ void VTerm::historyDisplay(bool absolute, s32 num)
 	if (!accel_scroll) {
 		requestUpdate(0, 0, width, height);
 	}
+	
+	draw_cursor();
 }
 
 u16 VTerm::get_line(u16 y)
