@@ -1,5 +1,6 @@
 /*
  *   Copyright  2008 dragchan <zgchan317@gmail.com>
+ *   This file is part of FbTerm.
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License
@@ -29,7 +30,6 @@ private:
 	virtual void readyRead(s8 *buf, u32 len);
 
 	u16 x, y;
-	u16 width, height;
 };
 
 #endif
